//
// Database.h
//////////////////////////////////////////////////////////////////////
// Database Record Declaration
//////////////////////////////////////////////////////////////////////

#ifndef __DATABASE_H__
#define __DATABASE_H__

//Database Name
#define DBNAME_EMPLOYEE		"EMPY"
#define DBNAME_ACCESSLOG	"TMLG"
#define DBNAME_DEPARTMENT	"DEPT"
#define DBNAME_TERMINALS	"SLAV"
#define DBNAME_EMPLOYEEFULL	"EMPF"
#define DBNAME_EMPLOYEESC	"XEMP"

// Record ID for iGuard Database Record Handle
typedef DWORD RCDID;
typedef RCDID *PRCDID;

#define TYPE_TIMERESTRICTION

#define DEPARTMENT_QUICKACCESS	"***"
#define DEPARTMENT_EVERYONE		"EVERYONE"

//Employee Record's Status Byte
#ifndef EMPLOYEE_ACTIVE
#define EMPLOYEE_ACTIVE			0x01
#define EMPLOYEE_ENROLLED		0x02
#define EMPLOYEE_USECARD		0x04
#define EMPLOYEE_AUTOMATCH		0x08
#define EMPLOYEE_USEPASSWORD	0x80
#define EMPLOYEE_INOUT			0x20	// 0010 0000
#define EMPLOYEE_INOUT_UNKNOWN	0x40	// 0100 0000
#endif //EMPLOYEE_ACTIVE

// Fingerprints system's constant
#define NUM_MINU 256					// Max number of minutiae likely to be found in a fingerprint
#define MINUSIZE 20						// Number of bytes per uncompressed minutia
#define COMPRESSED_MINUSIZE 4			// Number of bytes per compressed minutia
#define MAXMINUSIZE (NUM_MINU * COMPRESSED_MINUSIZE / sizeof(long))

typedef struct TIMERESTRICTION {
	DWORD Morning;		//00:00 to 11:59
	DWORD Afternoon;	//12:00 to 23:59
} TimeRestriction, *LPTimeRestriction;


//
// Employee Record (Smart Card)
//
typedef struct DB_SC_EMPLOYEE_T {
	char id[17];				// Employee ID
	char lastname[21];		// Last Name
	char firstname[21];		// First Name
	char othername[21];		// Other Name
	
	int status;				// Employee Status
	RCDID rcdid;				// Record's Object ID
	
	char password[17];			// Employee Password
	
	char branchcode[4];		// Branch Code
	DWORD smartcardid;		// Smart Card Serial No
	
} DB_SC_EMPLOYEE, *LPDB_SC_EMPLOYEE;

//
// Employee Record (minimum)
//
typedef struct DB_EMPLOYEE_T {		//Employee Record
	char id[17];
	char lastname[21];
	char firstname[21];
	char othername[21];
	DWORD status;					// Bit	 Descriptions
									//--------------------------------------------------------------
									//  0    Employee Active, 1 if employee actived
									//  1    Fingerpritns enrolled, 1 if employee already enrolled.
									//  3    User have smart card (for iGuard smart card version only)
									//  4    Enable Auto Match function
									// 2-14  Reserved.
									//  15   Use Password, 1 if employee have password.
	DWORD smartcardid;

	RCDID rcdid;
	char password[17];
	
} DB_EMPLOYEE, *LPDB_EMPLOYEE;

//
// Employee Record (full)
//
typedef struct DB_EMPLOYEEFULL_T {	//Employee Record, Fingerprints included
	
	char id[17];
	char lastname[21];
	char firstname[21];
	char othername[21];
	
	DWORD status;
	DWORD smartcardid;
	
	RCDID rcdid;
	char password[17];

	int numminutiae1;
	long minutiae1[MAXMINUSIZE];

	int numminutiae2;
	long minutiae2[MAXMINUSIZE];

} DB_EMPLOYEEFULL, *LPDB_EMPLOYEEFULL;

typedef struct DB_EMPLOYEE_T_X {	//Employee Search 
	int Status;
	char deptid[17];
	char lastname[21];
	char firstname[21];
} DB_EMPLOYEE_X, *LPDB_EMPLOYEE_X;

typedef struct DB_TIMELOG_T {		//Access Log
	char id[17];
	FILETIME logTime;
	char terminalID[17];
	int status;					// Record Status :	Bit 0 - 2
								//					0x00 IN
								//					0x01 OUT
								//					0x02 F1
								//					0x03 F2
								//					0x04 F3
								//					0x05 F4
								//					0x07 Extended Mode (Bit 8 - 11 Valid)
								//
								//					Bit 4 - 6
								//					0x00 : Not Used
								//					0x10 : Issue Card (Branch Code = SmartCard SN)
								//					0x20 : Enroll Fingerprints
								//
								//					Bit 7 : Unauthorized Entry
								//					
	RCDID rcdid;

} DB_TIMELOG, *LPDB_TIMELOG;

///////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Notes about "Log Status"
// 
// 1. LS Byte
// 
//       Bit# 7 6 5 4 3 2 1 0
//            | | | | | |   | 
//            | |  -| |  ----------- Normal Log Status
//            | |   | |                                0 0 0 - IN              LOGSTATUS_IN
//            | |   | |                                0 0 1 - OUT             LOGSTATUS_OUT
//            | |   | |                                0 1 0 - F1              LOGSTATUS_F1
//            | |   | |                                0 1 1 - F2              LOGSTATUS_F2
//            | |   | |                                1 0 0 - F3              LOGSTATUS_F3
//            | |   | |                                1 0 1 - F4              LOGSTATUS_F4
//            | |   | |                                1 1 0 - NOT USED
//            | |   | |                                1 1 1 - Extended Status LOGSTATUS_MASK
//            | |   | |
//            | |   |  ------------- Reserved, not used
//            | |   |
//            | |    --------------- Extended Log Status
//            | |                                      0 0 - Card in Bad List  EX_LOGSTATUS_CARDINBADLIST
//            | |                                      0 1 - Issue Card        EX_LOGSTATUS_ISSUECARD
//            | |                                      1 0 - Enrollment        EX_LOGSTATUS_ENROLLMENT
//            | |                                      1 1 - Set Password      EX_LOGSTATUS_SETPASSWORD
//            | |
//            |  ------------------- Single Access Group for DailySingleAccess EX_SINGLEACCESSGROUP
//            |
//             --------------------- Unauthorized Entry                        LOGSTATUS_UNAUTHOZIED
//


#define LOGSTATUS_IN			0x00
#define LOGSTATUS_OUT			0x01
#define LOGSTATUS_F1			0x02
#define LOGSTATUS_F2			0x03
#define LOGSTATUS_F3			0x04
#define LOGSTATUS_F4			0x05
#define LOGSTATUS_EX			0x07

#define LOGSTATUS_MASK			0x07
#define LOGSTATUS_EX_MASK		0x30
#define LOGSTATUS_UNAUTHOZIED	0x80

//
// Extended Time Log Status
//
#define EX_CARDINBADLIST		0x00
#define EX_ISSUECARD			0x10
#define EX_ENROLLMENT			0x20
#define EX_SETPASSWORD			0x30

#define EX_SINGLEACCESSGROUP	0x40	// NOTES : This is a BITWISE Status, therefore all status
										//         no. with bit 3 on can't be use anymore, i.e. 4, 5, 6... etc

#define EX_LOGSTATUS_ISSUECARD		(LOGSTATUS_EX | EX_ISSUECARD)
#define EX_LOGSTATUS_ENROLLMENT		(LOGSTATUS_EX | EX_ENROLLMENT)
#define EX_LOGSTATUS_SETPASSWORD	(LOGSTATUS_EX | EX_SETPASSWORD)
#define EX_LOGSTATUS_CARDINBADLIST	(LOGSTATUS_EX | EX_CARDINBADLIST)

//
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct DB_TIMELOG_T_X {		//Access Log Search
	char id[17];
	FILETIME From;
	FILETIME To;
	char TerminalID[17];
	char DeptID[17];
} DB_TIMELOG_X, *LPDB_TIMELOG_X;

typedef struct DB_DEPARTMENT_T {
	char id[17];
	char departmentname[31];
	int status;
	TimeRestriction restriction[8];
	RCDID rcdid;
} DB_DEPARTMENT, *LPDB_DEPARTMENT;

typedef struct DB_TERMINALS_T {
	char id[17];
	DWORD ip;
	int flat;
	DWORD reserved;
	char description[41];
	RCDID rcdid;
} DB_TERMINALS, *LPDB_TERMINALS;

typedef struct DB_ATTENDANCE_T {
	char szID[17];
	FILETIME inout[6];
	char isMore;
	RCDID rcdid_1;
	RCDID rcdid_2;
} DB_ATTENDANCE, *LPDB_ATTENDANCE;

//
// Notes for DB_INDENTIFY_INFO:
//---------------------------------------------------------------------
// 1. if numminutiae1 and mumminutiae2 are both equal to zero,
//    minutiaes holds password string (ASCII 0 - 9, A, B).
// 
typedef struct DB_INDENTIFY_INFO_T {
	char id[10];
	char firstname[16];
	char lastname[16];
	BYTE numminutiae1;
	BYTE numminutiae2;
	BYTE minutiaes[256];
} DB_INDENTIFY_INFO, *LPDB_INDENTIFY_INFO;

//
// Broad Cast Packets
//---------------------------------------------------------------------
//
typedef struct BROADCASTINFOEX_T {
	char Group;
	char Type;
	char HostName[20];
	DWORD SerialNo;
	DWORD IP;
	DWORD MasterIP;
	DWORD Version;
	DWORD Status; 
	char Description[41];
	char OtherInfo1[11];
	char OtherInfo2[11];
} BROADCASTINFOEX, *LPBROADCASTINFOEX;

//
// Bit mask for Status member of BROADCASTINFO Structure
//
#define STATUS_EXTENDED			0x80000000
#define STATUS_ISMASTER			0x00000001
#define STATUS_DHCP				0x00010000
#define STATUS_ALARM			0x10000000
#define STATUS_WEBDISABLED		0x01000000
#define STATUS_PATCHDISABLED	0x02000000
#define STATUS_SENDTWICE		0x04000000
#define STATUS_WEBSTRINGCHANGED	0x08000000
#define STATUS_DEMOUNIT			0x00100000
#define STATUS_FINGERPRINT		0x00200000
#define STATUS_SMARTCARD		0x00400000
#define STATUS_EXSERIALNO		0x000F0000

#endif //__DATABASE_H__


