
#ifndef __WinApr_h__
#define __WinApr_h__

/* ************************ winapr.h version 3 ******************/
/* 
   This file is used for porting between Windows and Unix  as well
   as to provide for the different DLL calling conventions 
   (cdecl & stdcall).
*/
/* ****************************************************************/

/* Put stuff here that only gets set once, independent of other definitions! */

/* To help detect cdecl vs. stdcall incompatibilities */
#ifndef WINAPR_VERSION
#define WINAPR_VERSION 2.0
#endif


/* These are abbreviations for Veridicom source-code compilation only */
#ifdef ESYS
#define EMBEDDED_SYSTEM
#endif
#ifdef FIXPT
#define FIXED_POINT_ONLY
#endif
#ifdef TTRIG
#define TABLE_LOOKUP_TRIG
#endif
#ifdef LITE
#define IMGPLUS_LITE
#endif
#ifdef SHARE
#define USE_SHARED_MEM
#endif
#if defined( _WIN32 ) && !defined( WIN32 )
#define WIN32
#endif
#if defined( _WIN32_WCE ) && !defined( WIN32 )
#define WIN32
#endif

/* Some useful definitions */
#ifndef DWORD
typedef unsigned long int	DWORD;
typedef DWORD				*LPDWORD;
#endif
#ifndef BYTE
typedef unsigned char		BYTE;
#endif
#ifndef WORD
typedef unsigned short		WORD;
#endif
#ifndef BOOL
typedef int 				BOOL;
#endif

#define FALSE	0
#define TRUE 	1
#define FAILURE	-1
#define SUCCESS	0


/* porting stuff */
/* Note: MSOFT16 implies 16-bit Microsoft OS -- Dos or Win16 */

#ifdef _DOS
  #if defined( EMBEDDED_SYSTEM )    
	#pragma message ("**OPTION** _DOS=Building for Embedded 16-bit DOS/NOS")
  #elif defined( FORCE_STATIC )
	#pragma message ("**OPTION** _DOS=Building for static 16-bit DOS/NOS")
  #else
	#pragma message ("**OPTION** _DOS=Building DLL for 16-bit DOS/NOS")
  #endif
  #define FARAPR __far
  #define HUGEAPR __huge
  #define MSOFT16
  #define INT32 long
#elif defined(WIN16)
  #pragma message ("**OPTION** WIN16=Building for 16-bit Windows")
  #if defined( EMBEDDED_SYSTEM )    
	#pragma message ("**OPTION** WIN16=Building for Embedded 16-bit Windows")
  #elif defined( FORCE_STATIC )
	#pragma message ("**OPTION** WIN16=Building for static 16-bit Windows")
  #else
	#pragma message ("**OPTION** WIN16=Building DLL for 16-bit Windows")
  #endif
  #define FARAPR __far
  #define HUGEAPR __huge
  #define MSOFT16
  #define INT32 long
  #ifndef WINDOWS
  #define WINDOWS
  #endif
#elif defined(WIN32)
  #if defined( EMBEDDED_SYSTEM )    
	#pragma message ("**OPTION** WIN32=Building for Embedded 32-bit Windows")
  #elif defined( FORCE_STATIC )
	#pragma message ("**OPTION** WIN32=Building for static 32-bit Windows")
  #else
	#pragma message ("**OPTION** WIN32=Building DLL for 32-bit Windows")
  #endif
  #define FARAPR 
  #define HUGEAPR
  #define COPYSIGN _copysign
  #define MILLISLEEP(x) _sleep(x)
  #define INT32 long
  #ifndef WINDOWS
  #define WINDOWS
  #endif
#elif defined(UNIX64)
  #if defined( EMBEDDED_SYSTEM )    
	#pragma message ("**OPTION** UNIX64=Building for Embedded 64-bit System")
  #elif defined( FORCE_STATIC )
	#pragma message ("**OPTION** UNIX64=Building for static 64-bit Unix")
  #else
	#pragma message ("**OPTION** UNIX64=Building DLL for 64-bit Unix")
  #endif
  #define FARAPR
  #define HUGEAPR
  #define COPYSIGN copysign
  #define MILLISLEEP(x) usleep(1000*(x))
  #define INT32 short
#else  /* Assume UNIX32 */
  #ifndef UNIX32
    #define UNIX32
  #endif
  #if defined( EMBEBBED_SYSTEM )    
	#pragma message ("**OPTION** NONE=Building for Embedded 32-bit System")
  #elif defined( FORCE_STATIC )
	#pragma message ("**OPTION** NONE=Building for static 32-bit Unix")
  #else
	#pragma message ("**OPTION** NONE=Building DLL for 32-bit Unix")
  #endif
  #define FARAPR
  #define HUGEAPR
  #define COPYSIGN copysign
  #define MILLISLEEP(x) usleep(1000*(x))
  #define INT32 long
#endif

#if defined(WIN32) || defined(MSOFT16)
  #ifndef WINDOWS
  #define WINDOWS
  #endif
  #define PATH_CHAR '\\'
#else
  #undef WINDOWS
  #define PATH_CHAR '/'
#endif


#else /* winapr already included ... */

/* 
Undefine all things if already defined. This is so header files each
time can re-include this file, and can tailor its macros for only
a specific header file independently of others. 
*/

#undef CDECL_DLL
#undef DECL_DLL
#undef CDECL_DLLIMP
#undef CDECL_DLLEXP
#undef DECL_DLLIMP
#undef DECL_DLLEXP
#undef CDECL_DLLEXP_DYN
#undef DECL_DLLEXP_DYN

#undef UCHARPP
#undef UCHARP

#endif /* __WinApr_h__ */


#if defined(WIN32) && !defined(FORCE_STATIC)
/********************** Windows NT/95 ************************/

#ifdef VDLLEXPTYPE

/*
   __stdcall allows us to support 32-bit Visual Basic, but wreaks
   havoc with other stuff, most notably exported function names.
   So, functions must be imported the same way they are exported
   for things to work. 

   However, most 3rd-party DLLs are cdecl, not stdcall, so care
   must be taken to use the right keyword in function prototypes.

   So, we use CDECL_DLL for cdecl and DECL_DLL for stdcall.
*/

#	define CDECL_DLLEXP(x) __declspec (dllexport) x 
#	define CDECL_DLLIMP(x) __declspec (dllimport) x 
#	define CDECL_DLLEXP_DYN(x,y) __declspec (dllexport) x ( *y ) 

#	if !defined( _WIN32_WCE ) 
//
//  Normal WINDOWS
//
#		define DECL_DLLEXP_DYN(x,y) __declspec (dllexport) x ( __stdcall *y ) 
#		define DECL_DLLEXP(x) __declspec (dllexport) x __stdcall 
#		define DECL_DLLIMP(x) __declspec (dllimport) x __stdcall 
#	else
#		define DECL_DLLEXP_DYN(x,y) __declspec (dllexport) x ( *y ) 
#		define DECL_DLLEXP(x) __declspec (dllexport) x 
#		define DECL_DLLIMP(x) __declspec (dllimport) x 
#	endif

#else //VDLLEXPTYPE

#	define CDECL_DLLEXP(x) x
#	define CDECL_DLLIMP(x) x
#	define CDECL_DLLEXP_DYN(x,y) x ( *y ) 
#	if !defined( _WIN32_WCE )	
//
//  Normal WINDOWS
//
#		define DECL_DLLEXP_DYN(x,y) x ( *y ) 
#		define DECL_DLLEXP(x) x	 __stdcall
#		define DECL_DLLIMP(x) x  __stdcall
#	else
#		define DECL_DLLEXP_DYN(x,y) x ( *y ) 
#		define DECL_DLLEXP(x) x 
#		define DECL_DLLIMP(x) x 
#	endif

#endif //VDLLEXPTYPE

#endif /* win32 */      

#if defined(WIN16) && !defined(FORCE_STATIC)
/********************** Windows 16 ************************/

/* No 16-bit Visual Basic __stdcall support. Sorry. */
#define CDECL_DLLEXP(x) x __export
#define CDECL_DLLIMP(x) x
#define CDECL_DLLEXP_DYN(x,y) __export x ( *y ) 
#define DECL_DLLEXP_DYN(x,y) __export x ( *y ) 
#define DECL_DLLEXP(x) x __export
#define DECL_DLLIMP(x) x

#endif /* win16 */

#if defined(UNIX32) || defined(UNIX64) || defined(_DOS) || defined(FORCE_STATIC)
/************************** UNIX!!! ***************************/

/* unix (SANE) case */
#define CDECL_DLLEXP(x) x
#define CDECL_DLLIMP(x) x
#define CDECL_DLLEXP_DYN(x,y) x ( *y ) 
#define DECL_DLLEXP_DYN(x,y) x ( *y ) 
#define DECL_DLLEXP(x) x
#define DECL_DLLIMP(x) x

#define _outp(port, data) outb(data, port)
#define _inp(port) inb(port)

#endif /* Unix */

/************************** Macros ****************************/

/* For making default be exports instead of imports */
#ifdef VDLLEXPTYPE
#define CDECL_DLL(x) CDECL_DLLEXP(x)
#define DECL_DLL(x) DECL_DLLEXP(x)
#else
#define CDECL_DLL(x) CDECL_DLLIMP(x)
#define DECL_DLL(x) DECL_DLLIMP(x)
#endif

//
// ************* NOTES *************
//
// For use with Microsoft Windows
//
//   DECL_DLL(BOOL) Func(int nParam);
// 
// is equivalent to,
//
//   BOOL __stdcall Func(int nParam); 
//
//